@@ -1,12 +1,15 @@
 package Data::Random::String::Matches;
 
+use 5.010;
+
 use strict;
 use warnings;
 
-use Carp qw(croak);
+use Carp qw(carp croak);
 use Params::Get;
+use utf8;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 NAME
 
@@ -14,26 +17,46 @@ Data::Random::String::Matches - Generate random strings matching a regex
 
 =head1 SYNOPSIS
 
-    use Data::Random::String::Matches;
+	use Data::Random::String::Matches;
 
-    # Create generator with regex and optional length
-    my $gen = Data::Random::String::Matches->new(qr/[A-Z]{3}\d{4}/, 7);
+	# Create a generator with regex and optional length
+	my $gen = Data::Random::String::Matches->new(qr/[A-Z]{3}\d{4}/, 7);
 
-    # Generate a matching string
-    my $str = $gen->generate();
-    print $str;  # e.g., "XYZ1234"
+	# Generate a matching string
+	my $str = $gen->generate();
+	print $str;  # e.g., "XYZ1234"
 
-    # Alternation
-    my $gen2 = Data::Random::String::Matches->new(qr/(cat|dog|bird)/);
-    my $animal = $gen2->generate_smart();  # "cat", "dog", or "bird"
+	# Alternation
+	my $gen2 = Data::Random::String::Matches->new(qr/(cat|dog|bird)/);
+	my $animal = $gen2->generate_smart();  # "cat", "dog", or "bird"
 
-    # Backreferences
-    my $gen3 = Data::Random::String::Matches->new(qr/(\w{3})-\1/);
-    my $str3 = $gen3->generate_smart();  # e.g., "abc-abc"
+	# Backreferences
+	my $gen3 = Data::Random::String::Matches->new(qr/(\w{3})-\1/);
+	my $str3 = $gen3->generate_smart();  # e.g., "abc-abc"
 
-    # Groups and quantifiers
-    my $gen4 = Data::Random::String::Matches->new(qr/(ha){2,4}/);
-    my $laugh = $gen4->generate_smart();  # "haha", "hahaha", or "hahahaha"
+	# Groups and quantifiers
+	my $gen4 = Data::Random::String::Matches->new(qr/(ha){2,4}/);
+	my $laugh = $gen4->generate_smart();  # "haha", "hahaha", or "hahahaha"
+
+	# Unicode
+	$gen = Data::Random::String::Matches->new(qr/\p{L}{5}/);
+
+	# Named captures
+	$gen = Data::Random::String::Matches->new(qr/(?<year>\d{4})-\k<year>/);
+
+	# Possessive
+	$gen = Data::Random::String::Matches->new(qr/\d++[A-Z]/);
+
+	# Lookaheads
+	$gen = Data::Random::String::Matches->new(qr/\d{3}(?=[A-Z])/);
+
+	# Combined
+	$gen = Data::Random::String::Matches->new(
+    		qr/(?<prefix>\p{Lu}{2})\d++\k<prefix>(?=[A-Z])/
+	);
+
+	# Consistency with Legacy software
+	print Data::Random::String::Matches->create_random_string(length => 3, regex => '\d{3}'), "\n";
 
 =head1 DESCRIPTION
 
@@ -135,19 +158,35 @@ a wide range of regex features.
 
 =back
 
+=head1 LIMITATIONS
+
+=over 4
+
+=item * Lookaheads and lookbehinds ((?=...), (?!...)) are not supported
+
+=item * Named groups ((?<name>...)) are not supported
+
+=item * Possessive quantifiers (*+, ++) are not supported
+
+=item * Unicode properties (\p{L}, \p{N}) are not supported
+
+=item * Some complex nested patterns may not work correctly with smart parsing
+
+=back
+
 =head1 EXAMPLES
 
-    # Email-like pattern
-    my $gen = Data::Random::String::Matches->new(qr/[a-z]+@[a-z]+\.com/);
+	# Email-like pattern
+	my $gen = Data::Random::String::Matches->new(qr/[a-z]+@[a-z]+\.com/);
 
-    # API key pattern
-    my $gen = Data::Random::String::Matches->new(qr/^AIza[0-9A-Za-z_-]{35}$/);
+	# API key pattern
+	my $gen = Data::Random::String::Matches->new(qr/^AIza[0-9A-Za-z_-]{35}$/);
 
-    # Phone number
-    my $gen = Data::Random::String::Matches->new(qr/\d{3}-\d{3}-\d{4}/);
+	# Phone number
+	my $gen = Data::Random::String::Matches->new(qr/\d{3}-\d{3}-\d{4}/);
 
-    # Repeated pattern
-    my $gen = Data::Random::String::Matches->new(qr/(\w{4})-\1/);
+	# Repeated pattern
+	my $gen = Data::Random::String::Matches->new(qr/(\w{4})-\1/);
 
 =head1 METHODS
 
@@ -161,7 +200,7 @@ C<$length> is optional and defaults to 10 (used for fallback generation).
 sub new {
 	my ($class, $regex, $length) = @_;
 
-	croak "Regex pattern is required" unless defined $regex;
+	croak 'Regex pattern is required' unless defined $regex;
 
 	# Convert string to regex if needed
 	my $regex_obj = ref($regex) eq 'Regexp' ? $regex : qr/$regex/;
@@ -171,6 +210,7 @@ sub new {
 		regex_str => "$regex",
 		length	=> $length || 10,
 		backrefs	=> {},  # Store backreferences
+		named_refs  => {},	   # Store named captures
 	};
 
 	return bless $self, $class;
@@ -254,16 +294,57 @@ sub generate_smart {
 	return $self->_build_from_pattern($self->{regex_str});
 }
 
+=head2 generate_many($count, $unique)
+
+Generate a number of (possibly) unique strings for the regex
+
+=cut
+
+sub generate_many {
+	my ($self, $count, $unique) = @_;
+
+	croak 'Count must be a positive integer' unless defined $count && $count > 0;
+
+	my @results;
+
+	if ($unique) {
+		# Generate unique strings
+		my %seen;
+		my $attempts = 0;
+		my $max_attempts = $count * 100;  # Reasonable limit
+
+		while (keys %seen < $count && $attempts < $max_attempts) {
+			my $str = $self->generate();
+			$seen{$str} = 1;
+			$attempts++;
+		}
+
+		if (keys %seen < $count) {
+			carp 'Only generated ', (scalar keys %seen), " unique strings out of $count requested";
+		}
+
+		@results = keys %seen;
+	} else {
+		# Generate any strings (may have duplicates)
+		push @results, $self->generate() for (1 .. $count);
+	}
+
+	return @results;
+}
+
 sub _build_from_pattern {
 	my ($self, $pattern) = @_;
 
 	# Reset backreferences for each generation
 	$self->{backrefs} = {};
+	$self->{named_refs} = {};
 	$self->{group_counter} = 0;
 
 	# Remove regex delimiters and modifiers
 	# Handle (?^:...), (?i:...), (?-i:...) etc
 	$pattern =~ s/^\(\?\^?[iumsx-]*:(.*)\)$/$1/;
+
+	# Remove anchors (they're handled by the regex match itself)
 	$pattern =~ s/^\^//;
 	$pattern =~ s/\$//;
 
@@ -293,8 +374,27 @@ sub _parse_sequence {
 				} else {
 					croak "Backreference \\$ref_num used before group defined";
 				}
+			} elsif ($next eq 'k' && substr($pattern, $i+1, 1) eq '<') {
+				# Named backreference \k<name>
+				my $end = index($pattern, '>', $i+2);
+				my $name = substr($pattern, $i+2, $end-$i-2);
+				if (exists $self->{named_refs}{$name}) {
+					$result .= $self->{named_refs}{$name};
+				} else {
+					croak "Named backreference \\k<$name> used before group defined";
+				}
+				$i = $end;
+			} elsif ($next eq 'p' && substr($pattern, $i+1, 1) eq '{') {
+				# Unicode property \p{L}, \p{N}, etc.
+				my $end = index($pattern, '}', $i+2);
+				my $prop = substr($pattern, $i+2, $end-$i-2);
+				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $end, sub {
+					$self->_unicode_property_char($prop);
+				});
+				$result .= $generated;
+				$i = $new_i;
 			} elsif ($next eq 'd') {
-				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub { int(rand(10)) });
+				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub { int(rand(10)) }, 1);
 				$result .= $generated;
 				$i = $new_i;
 			} elsif ($next eq 'w') {
@@ -335,26 +435,76 @@ sub _parse_sequence {
 		} elsif ($char eq '[') {
 			# Character class
 			my $end = $self->_find_matching_bracket($pattern, $i);
-			croak "Unmatched [" if $end == -1;
+			croak 'Unmatched [' if $end == -1;
 
 			my $class = substr($pattern, $i+1, $end-$i-1);
 			my ($generated, $new_i) = $self->_handle_quantifier($pattern, $end, sub {
 				$self->_random_from_class($class);
-			});
+			}, 1);
 			$result .= $generated;
 			$i = $new_i + 1;
 		} elsif ($char eq '(') {
-			# Group
+			# Group - could be various types
 			my $end = $self->_find_matching_paren($pattern, $i);
-			croak "Unmatched (" if $end == -1;
+			croak 'Unmatched (' if $end == -1;
 
 			my $group_content = substr($pattern, $i+1, $end-$i-1);
 
-			# Check for non-capturing group
+			# Check for special group types
 			my $is_capturing = 1;
+			my $is_lookahead = 0;
+			my $is_lookbehind = 0;
+			my $is_negative = 0;
+			my $group_name = undef;
+
 			if ($group_content =~ /^\?:/) {
+				# Non-capturing group
 				$is_capturing = 0;
 				$group_content = substr($group_content, 2);
+			} elsif ($group_content =~ /^\?<([^>]+)>/) {
+				# Named capture (?<name>...)
+				$group_name = $1;
+				$group_content = substr($group_content, length($1) + 3);
+			} elsif ($group_content =~ /^\?=/) {
+				# Positive lookahead (?=...)
+				$is_lookahead = 1;
+				$is_capturing = 0;
+				$group_content = substr($group_content, 2);
+			} elsif ($group_content =~ /^\?!/) {
+				# Negative lookahead (?!...)
+				$is_lookahead = 1;
+				$is_negative = 1;
+				$is_capturing = 0;
+				$group_content = substr($group_content, 2);
+			} elsif ($group_content =~ /^\?<=/) {
+				# Positive lookbehind (?<=...)
+				$is_lookbehind = 1;
+				$is_capturing = 0;
+				$group_content = substr($group_content, 3);
+			} elsif ($group_content =~ /^\?<!/) {
+				# Negative lookbehind (?<!...)
+				$is_lookbehind = 1;
+				$is_negative = 1;
+				$is_capturing = 0;
+				$group_content = substr($group_content, 3);
+			}
+
+			# Handle lookaheads/lookbehinds
+			if ($is_lookahead) {
+				# For positive lookahead, generate the pattern but don't advance
+				# For negative lookahead, avoid the pattern
+				if (!$is_negative) {
+					# Generate what the lookahead expects but don't consume it
+					# This is a simplification - we just note the constraint
+				}
+				# Lookaheads don't add to the result
+				$i = $end + 1;
+				next;
+			} elsif ($is_lookbehind) {
+				# Lookbehinds check what came before
+				# For generation, we can mostly ignore them
+				$i = $end + 1;
+				next;
 			}
 
 			# Check for alternation
@@ -369,10 +519,14 @@ sub _parse_sequence {
 			if ($is_capturing) {
 				$self->{group_counter}++;
 				$self->{backrefs}{$self->{group_counter}} = $generated;
+
+				if (defined $group_name) {
+					$self->{named_refs}{$group_name} = $generated;
+				}
 			}
 
-			# Handle quantifier after group
-			my ($final_generated, $new_i) = $self->_handle_quantifier($pattern, $end, sub { $generated });
+			# Handle quantifier after group (including possessive)
+			my ($final_generated, $new_i) = $self->_handle_quantifier($pattern, $end, sub { $generated }, 1);
 			$result .= $final_generated;
 			$i = $new_i + 1;
 		} elsif ($char eq '.') {
@@ -389,7 +543,7 @@ sub _parse_sequence {
 			last;
 		} elsif ($char =~ /[+*?]/ || $char eq '{') {
 			# Quantifier without preceding element - shouldn't happen in valid regex
-			croak "Quantifier $char without preceding element";
+			croak "$pattern: Quantifier '$char' without preceding element";
 		} elsif ($char =~ /[\w ]/) {
 			# Literal character
 			my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub { $char });
@@ -406,14 +560,31 @@ sub _parse_sequence {
 }
 
 sub _handle_quantifier {
-	my ($self, $pattern, $pos, $generator) = @_;
+	my ($self, $pattern, $pos, $generator, $check_possessive) = @_;
+	$check_possessive //= 1;  # Default to checking for possessive
 
 	my $next = substr($pattern, $pos + 1, 1);
+	my $is_possessive = 0;
+
+	# Check for possessive quantifier (+)
+	if ($check_possessive && $pos + 2 < length($pattern)) {
+		my $after_next = substr($pattern, $pos + 2, 1);
+		if (($next =~ /[+*?]/ || $next eq '}') && $after_next eq '+') {
+			$is_possessive = 1;
+		}
+	}
 
 	if ($next eq '{') {
-		my $end = index($pattern, '}', $pos);
+		my $end = index($pattern, '}', $pos + 2);
+		croak "Unmatched '}'" if ($end == -1);
 		my $quant = substr($pattern, $pos + 2, $end - $pos - 2);
 
+		# Check for possessive after }
+		if ($check_possessive && $end + 1 < length($pattern) && substr($pattern, $end + 1, 1) eq '+') {
+			$is_possessive = 1;
+			$end++;
+		}
+
 		my $result = '';
 		if ($quant =~ /^(\d+)$/) {
 			# Exact: {n}
@@ -429,21 +600,33 @@ sub _handle_quantifier {
 		}
 		return ($result, $end);
 	} elsif ($next eq '+') {
-		# One or more
+		# One or more (possessive: ++)
+		my $actual_end = $pos + 1;
+		if ($is_possessive) {
+			$actual_end++;
+		}
 		my $count = 1 + int(rand(5));
 		my $result = '';
 		$result .= $generator->() for (1 .. $count);
-		return ($result, $pos + 1);
+		return ($result, $actual_end);
 	} elsif ($next eq '*') {
-		# Zero or more
+		# Zero or more (possessive: *+)
+		my $actual_end = $pos + 1;
+		if ($is_possessive) {
+			$actual_end++;
+		}
 		my $count = int(rand(6));
 		my $result = '';
 		$result .= $generator->() for (1 .. $count);
-		return ($result, $pos + 1);
+		return ($result, $actual_end);
 	} elsif ($next eq '?') {
-		# Zero or one
+		# Zero or one (possessive: ?+)
+		my $actual_end = $pos + 1;
+		if ($is_possessive) {
+			$actual_end++;
+		}
 		my $result = rand() < 0.5 ? $generator->() : '';
-		return ($result, $pos + 1);
+		return ($result, $actual_end);
 	} else {
 		# No quantifier
 		return ($generator->(), $pos);
@@ -539,6 +722,12 @@ sub _random_from_class {
 				push @chars, ('a'..'z', 'A'..'Z', '0'..'9', '_');
 			} elsif ($next eq 's') {
 				push @chars, (' ', "\t", "\n");
+			} elsif ($next eq 'p' && substr($class, $i+1, 1) eq '{') {
+				# Unicode property in character class
+				my $end = index($class, '}', $i+2);
+				my $prop = substr($class, $i+2, $end-$i-2);
+				push @chars, $self->_unicode_property_chars($prop);
+				$i = $end;
 			} else {
 				push @chars, $next;
 			}
@@ -546,7 +735,7 @@ sub _random_from_class {
 			# Range
 			my $end = substr($class, $i+2, 1);
 			push @chars, ($char .. $end);
-			$i += 2;
+			$i += 2;	# Will be incremented again by loop, total +3
 		} else {
 			push @chars, $char;
 		}
@@ -561,11 +750,52 @@ sub _random_from_class {
 	return @chars ? $chars[int(rand(@chars))] : 'X';
 }
 
+sub _unicode_property_char {
+	my ($self, $prop) = @_;
+	my @chars = $self->_unicode_property_chars($prop);
+	return @chars ? $chars[int(rand(@chars))] : 'X';
+}
+
+sub _unicode_property_chars {
+	my ($self, $prop) = @_;
+
+	# Common Unicode properties
+	if ($prop eq 'L' || $prop eq 'Letter') {
+		# Letters, skip × and ÷ which are symbols
+		return ('a' .. 'z', 'A' .. 'Z', map { chr($_) } ((ord'À')..ord('Ö'), ord('Ø')..ord('ö'), ord('ø')..ord('ÿ')));
+	} elsif ($prop eq 'N' || $prop eq 'Number') {
+		# Numbers
+		# return ('0' .. '9', map { chr($_) } (ord('①').. ord('⑳')));
+		return ('0' .. '9');
+	} elsif ($prop eq 'Lu' || $prop eq 'Uppercase_Letter') {
+		# Uppercase letters, skip × which is not a letter
+		return ('A' .. 'Z', map { chr($_) } (ord('À') .. ord('Ö'), ord('ø') .. ord('Þ')));
+	} elsif ($prop eq 'Ll' || $prop eq 'Lowercase_Letter') {
+		# Lowercase letters, skip ÷ which is not a letter
+		return ('a' .. 'z', map { chr($_) } (ord('à') .. ord('ö'), ord('ø') .. ord('ÿ')));
+	} elsif ($prop eq 'P' || $prop eq 'Punctuation') {
+		# Punctuation
+		return ('.', ',', '!', '?', ';', ':', '-', '—', '…');
+	} elsif ($prop eq 'S' || $prop eq 'Symbol') {
+		# Symbols
+		return ('$', '€', '£', '¥', '©', '®', '™', '°', '±', '×', '÷');
+	} elsif ($prop eq 'Z' || $prop eq 'Separator') {
+		# Separators
+		return (' ', "\t", "\n");
+	} elsif ($prop eq 'Nd' || $prop eq 'Decimal_Number') {
+		# Decimal numbers
+		return ('0'..'9');
+	} else {
+		# Unknown property - return letters as default
+		return ('a'..'z', 'A'..'Z');
+	}
+}
+
 =head2 create_random_string
 
 For consistency with L<Data::Random::String>.
 
-  print Data::Random::String->create_random_string(length => 3, regex => '\d{3}'), "\n";
+  print Data::Random::String::Matches->create_random_string(length => 3, regex => '\d{3}'), "\n";
 
 =cut
 
